/*  XRoar - a Dragon/Tandy Coco emulator
 *  Copyright (C) 2003-2005  Ciaran Anscomb
 *
 *  See COPYING for redistribution conditions. */

#ifndef __MACHINE_H__
#define __MACHINE_H__

#include "types.h"

/* Dragon 64s and later Dragon 32s used a 14.218MHz crystal
 * (ref: "Dragon 64 differences", Graham E. Kinns and then a motherboard) */
#define OSCILLATOR_RATE 14218000

/* NTSC timings: */
#define CYCLES_PER_SCANLINE 910
#define ACTIVE_SCANLINES_PER_FRAME 256
#define TOTAL_SCANLINES_PER_FRAME 262
#define TOP_BORDER_OFFSET 14

/* NTSC kludged timings: */
/*
#define CYCLES_PER_SCANLINE 910
#define ACTIVE_SCANLINES_PER_FRAME 262
#define TOTAL_SCANLINES_PER_FRAME 310
#define TOP_BORDER_OFFSET 25
*/

#define CYCLES_PER_FRAME (CYCLES_PER_SCANLINE * TOTAL_SCANLINES_PER_FRAME)

#define CPU_RATE_DIVISOR 16
#define CPU_RATE (OSCILLATOR_RATE / CPU_RATE_DIVISOR)

#define NUM_MACHINES 3
#define DRAGON64 0
#define COCO 1
#define DRAGON32 2

#define IS_DRAGON64 (machine == DRAGON64)
#define IS_DRAGON32 (machine == DRAGON32)
/* Quicker test, but may need changing if there are more machines: */
#define IS_DRAGON (!IS_COCO)
#define IS_COCO (machine == COCO)

#define NUM_KEYBOARDS 2
#define DRAGON_KEYBOARD 0
#define COCO_KEYBOARD 1

#define IS_DRAGON_KEYBOARD (machine_keyboard == DRAGON_KEYBOARD)
#define IS_COCO_KEYBOARD (machine_keyboard == COCO_KEYBOARD)

typedef struct { uint_fast8_t col, row; } Key;
typedef Key Keyboard[128];

extern int machine, machine_keyboard;
extern int dragondos_enabled;

/* ram0 is the lower bank of RAM on all machines
 * ram1 is the upper bank of RAM on 64K machines
 * rom0 is the ROM for all machines
 * rom1 is the alternate ROM for Dragon 64 (upper 16K is copied from rom0) */
extern uint8_t ram0[0x8000];
extern uint8_t ram1[0x8000];
extern uint8_t rom0[0x8000];
extern uint8_t rom1[0x8000];

/* The current keyboard layout is copied into this array of 128 'Key's,
 * indexed by ASCII value of the key with the following special meanings:
 * {8,8} means no key exists with that code
 * Code 0 is used for Shift, Code 27 (ASCII 'escape') used for Break */
extern Keyboard keyboard;

/* Tape routines still work by intercepting ROM calls - these breakpoints
 * are set according to which ROM is loaded: */
extern int_fast32_t brk_csrdon, brk_bitin;

void machine_init(void);
void machine_reset(int hard);
/* Selects a machine architecture, loads the relevant ROMs and resets */
void machine_set(int mode);
/* Selects a keyboard layout (it's sometimes useful to select an alternate
 * layout without resetting the machine). */
void machine_set_keyboard(int mode);

#endif  /* __MACHINE_H__ */
