/*  XRoar - a Dragon/Tandy Coco emulator
 *  Copyright (C) 2003-2005  Ciaran Anscomb
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "config.h"

#ifdef HAVE_NULL_AUDIO

#include "types.h"
#include "logging.h"

#include <errno.h>
#include <fcntl.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/ioctl.h>
#include <linux/rtc.h>

#include "events.h"
#include "sound.h"
#include "pia.h"
#include "xroar.h"

static int init(void);
static void shutdown(void);
static void reset(void);
static void update(void);

SoundModule sound_null_module = {
	NULL,
	"null",
	"NULL audio (silence)",
	init, shutdown, reset, update
};

#define SAMPLE_RATE 64
#define FRAME_CYCLES ((uint32_t)(OSCILLATOR_RATE / SAMPLE_RATE))

int fd;

static void flush_frame(void);
static event_t flush_event = {0, flush_frame, 0, NULL };

static int init(void) {
	LOG_DEBUG(2,"Initialising NULL audio driver\n");
	if ((fd = open ("/dev/rtc", O_RDONLY)) == -1) {
		LOG_ERROR("Couldn't open /dev/rtc\n");
		return 1;
	}
	if (ioctl(fd, RTC_IRQP_SET, SAMPLE_RATE) == -1) {
		LOG_ERROR("Couldn't set %dHz clock\n", SAMPLE_RATE);
		close(fd);
		return 1;
	}
	if (ioctl(fd, RTC_PIE_ON, 0) == -1) {
		LOG_ERROR("Couldn't enable periodic interrupts\n");
		close(fd);
		return 1;
	}
	return 0;
}

static void shutdown(void) {
	LOG_DEBUG(2,"Shutting down NULL audio driver\n");
	if (ioctl(fd, RTC_PIE_OFF, 0) == -1) {
		LOG_WARN("Couldn't disable periodic interrupts\n");
	}
	close(fd);
}

static void reset(void) {
	flush_event.at_cycle = current_cycle + FRAME_CYCLES;
	event_schedule(&flush_event);
}

static void update(void) {
}
void flush_frame(void) {
	unsigned long data;
	read(fd, &data, sizeof(unsigned long));
	flush_event.at_cycle += FRAME_CYCLES;
	event_schedule(&flush_event);
}

#endif  /* HAVE_NULL_AUDIO */
