/*  XRoar - a Dragon/Tandy Coco emulator
 *  Copyright (C) 2003-2004  Ciaran Anscomb
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <stdio.h>
#include <stdlib.h>
#include <SDL.h>
#include <SDL_image.h>

void getpixel(SDL_Surface *surface, int x, int y, Uint8 *r, Uint8 *g, Uint8 *b);

int main(int argc, char **argv) {
	Uint32 p;
	Uint8 r,g,b;
	int char_width, char_height;
	int x, y, i, j, bits;
	Uint32 out;
	SDL_Surface *in;
	if (argc < 3) {
		printf("Usage: %s name infile\n", argv[0]);
		exit(1);
	}
	in = IMG_Load(argv[2]);
	char_width = (in->w)/32;
	char_height = (in->h)/3;
	printf("/* This file is automatically generated by font2c. */\n\n");
	printf("#include \"video.h\"\n#include \"types.h\"\n\n");
	printf("uint32_t %s[] = {\n", argv[1]);
	for (y = 0; y < 3; y++) {
		for (x = 0; x < 32; x++) {
			int c = 32+(y*32)+x;
			if (c < 127)
				printf("\t/* '%c' */\n", 32+(y*32)+x);
			else
				printf("\t/* DEL */\n");
			out = 0;
			printf("\t");
			for (i = 0; i < char_width; i++) {
				for (j = 0; j < char_height; j++) {
					out <<= 1;
					getpixel(in, (x*char_width)+i, (y*char_height)+j, &r, &g, &b);
					if (r > 127 && g > 127 && b > 127)
						out |= 1;
				}
				printf("0x%08x, ", out);
			}
			printf("\n");
		}
	}
	SDL_FreeSurface(in);
	printf("};\n");
	return 0;
}

void getpixel(SDL_Surface *surface, int x, int y, Uint8 *r, Uint8 *g, Uint8 *b)
{
	int bpp;
	Uint8 *p;
	Uint32 pixel = 0;
	SDL_LockSurface(surface);
	bpp = surface->format->BytesPerPixel;
	p = (Uint8 *)surface->pixels + y * surface->pitch + x * bpp;
	switch(bpp) {
		case 1: pixel = *p; break;
		case 2: pixel = *(Uint16 *)p; break;
		case 3:
			if (SDL_BYTEORDER == SDL_BIG_ENDIAN)
				pixel = p[0] << 16 | p[1] << 8 | p[2];
			else
				pixel = p[0] | p[1] << 8 | p[2] << 16;
			break;
		case 4: pixel = *(Uint32 *)p; break;
		default: break;
	}
	SDL_UnlockSurface(surface);
	SDL_GetRGB(pixel, surface->format, r, g, b);
}
