/*  XRoar - a Dragon/Tandy Coco emulator
 *  Copyright (C) 2003-2005  Ciaran Anscomb
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "config.h"
#include "types.h"

#include "events.h"
#include "video.h"
#include "m6809.h"
#include "sam.h"
#include "vdg.h"
#include "pia.h"
#include "logging.h"

#ifndef GP32
#include "vdg_bitmaps.c"
#endif

static void generate_hsync(void);
static void generate_vsync(void);
static event_t hsync_event = { 0, generate_hsync, 0, NULL };
static event_t vsync_event = { 0, generate_vsync, 0, NULL };
int vdg_call;

void (*vdg_render_scanline)(void);

void vdg_init(void) {
	video_artifact_mode = 0;
	vdg_render_scanline = video_module->vdg_render_sg4;
	vdg_call = 0;
	hsync_event.at_cycle = CYCLES_PER_SCANLINE;
	event_schedule(&hsync_event);
}

void vdg_reset(void) {
	video_module->vdg_reset();
	vdg_set_mode();
}

void vdg_vsync(void) {
	video_module->vdg_vsync();
	sam_vdg_fsync();
}

static void generate_hsync(void) {
	if (vdg_call >= TOP_BORDER_OFFSET) {
		if (vdg_call < (24 + TOP_BORDER_OFFSET)) {
#ifndef HAVE_GP32
			video_module->render_border();
#endif
		} else {
			if (vdg_call < (216 + TOP_BORDER_OFFSET)) {
#ifdef HAVE_GP32
				if ((vdg_call & 3) == 0)
#endif
					vdg_render_scanline();
			} else {
				if (vdg_call < (240 + TOP_BORDER_OFFSET)) {
#ifndef HAVE_GP32
					video_module->render_border();
#endif
				}
			}
		}
	}
	PIA_SET_P0CA1;
	vdg_call++;
	if (vdg_call == ACTIVE_SCANLINES_PER_FRAME) {
		//vdg_vsync();
		sam_vdg_fsync();
		PIA_SET_P0CB1;
	}
	if (vdg_call >= TOTAL_SCANLINES_PER_FRAME) {
		video_module->vdg_vsync();  // XXX
		vdg_call = 0;
	}
	hsync_event.at_cycle += CYCLES_PER_SCANLINE;
	event_schedule(&hsync_event);
}

static void generate_vsync(void) {
}

void vdg_set_mode(void) {
	uint_fast8_t mode = PIA_1B.port_output;
	/* Update video module */
	video_module->vdg_set_mode(mode);
	switch ((mode & 0xf0) >> 4) {
		case 0: case 2: case 4: case 6:
			vdg_render_scanline = video_module->vdg_render_sg4;
			break;
		case 1: case 3: case 5: case 7:
			vdg_render_scanline = video_module->vdg_render_sg6;
			break;
		case 8:
			vdg_render_scanline = video_module->vdg_render_cg1;
			break;
		case 9: case 11: case 13:
			vdg_render_scanline = video_module->vdg_render_rg1;
			break;
		case 10: case 12: case 14:
			vdg_render_scanline = video_module->vdg_render_cg2;
			break;
		case 15: default:
			if (video_artifact_mode) {
				vdg_render_scanline = video_module->vdg_render_cg2;
			} else {
				vdg_render_scanline = video_module->vdg_render_rg6;
			}
			break;
	}
}
